from util import *

class Action: 
    def __init__(self, desc, direction, friends):
        self.desc = desc
        self.direction = direction
        self.friends = friends
        actions[desc] = self
        

# examples
Action('read', 'in', [])
Action('write', 'out', ['create'])
Action('activate', 'out', [])
Action('spawn', 'in', [])
Action('open', 'out', [])
Action('exec', 'out', [])

class Resource:
    def __init__(self, name, id, parent = None):
        #print "Creating " + self.__class__.__name__ + " " + name
        self.name = name
        self.id = id
        self.is_distinguished_wrt = []
        self.roles = []
        self.in_edges = [] # set of Transitions
        self.out_edges = [] # set of Transitions
        resources_dict[id] = self
        trans[self] = {}
        self.processes = {}
        self.pipes = {}
        self.sending_to = []
        self.receiving_from = []
        self.failures = []
        self.original_context = None
        self.children = []
        self.executing = []
        self.executed_by = []
        self.context_assignment_status = 'default'
        self.security_context = None
        self.alt_assignment = None
        self.is_exec_t = 0
        self.parent = parent # e.g., directory containing a file
        if parent != None:
            parent.children.append(self)
       

            
class Process (Resource):
    def __repr__(self):
        return "<Process " + self.name + ">"
    def add_transition(self, action, holder_id):
        #error if going from a file to a file
        if self.__class__.__name__ == 'File' and self == resources_dict[holder_id]:
            print "ERROR" 
        actobj = actions[action]
        direction = actobj.direction
        #print "creating " + action + ":" + holder_id
##        add = 1
##        if direction == 'out': # writing
##            trans = self.out_edges
##            for t in trans:
##                if t.action == action and \
##                   t.range == resources_dict[holder_id]:
##                    add = 0
##                    continue
##        elif direction == 'in': # reading
##            trans = self.in_edges
##            for t in trans:
##                if t.action == action and \
##                   t.domain == resources_dict[holder_id]:
##                    add = 0
##                    continue
##        if 1: #add == 1:
        Transition(self, action, resources_dict[holder_id])
        
    def compute_out_processes(self):
        """ Identifies process-to-process interactions"""
        for t in self.out_edges:
            resource = t.range
            if resource.__class__.__name__  == 'Process':
                self.executing.append(resource)
                resource.executed_by.append(self)
            else:            
                for rt in resource.out_edges:
                    target = rt.range
                    if target != self:
                        self.sending_to.append(rt.range)
                        rt.range.receiving_from.append(self)
    def gen_allow(self):
        """ Preliminary thoughts on generating rules"""
        for t in self.out_edges:
            act = actions[t.action]
            allowables = act.friends + [act.desc]
            holder = t.range
            parent = holder.parent
            print "allow " + self.name + "_t " + parent.name \
                  + "_t:" + holder.__class__.__name__ \
                  +  strip_commas(allowables)

class Holder (Resource):
    def __repr__(self):
        return "<Holder " + self.name + ">"

class Fifo_file (Resource):
    def __init__(self, name, id, creator, nbrs):
        #print "Creating " + self.__class__.__name__ + " " + name
        prgm = get_current_program()
        self.name = name
        self.id = id
        self.nbrs = nbrs
        self.creator = creator
        creator.pipes[id] = self
        self.is_distinguished_wrt = []
        self.roles = []
        self.in_edges = [] # set of Transitions
        self.out_edges = [] # set of Transitions
        resources_dict[id] = self
        trans[self] = {}
        self.context_assignment_status = 'new'
        self.security_context = 'self'
        self.original_context = 'self'
    def __repr__(self):
        return "<Pipe " + self.name + ">"
    
class Dir(Resource):
    def __repr__(self):
        parent = ""
        if self.parent != None:
            parent = self.parent.name
        return "<Dir" + parent + ": " + self.name + ">" 

class File(Resource):
    def __repr__(self):
        return "<File " +  self.name + ">" 

class SharedMemory (Resource):
    def __repr__(self):
        return "<Shared Memory " + self.name + ">" 

class Socket (Resource):
    def __repr__(self):
        return "<Socket " + self.name + ">"

class Spawn (Resource):
    def __repr__(self):
        return "<Spawn " + self.name + ">"
    
class Transition:
    def __init__(self, process, action, holder):
        self.action = action #'read','write', etc
        self.actobj = actions[action]
        direction = self.actobj.direction
        self.meanings = []
        if direction == 'out': # writing
            self.domain = process
            self.range = holder
            process.out_edges.append(self)
            holder.in_edges.append(self)
        elif direction == 'in': # reading
            self.domain = holder
            self.range = process
            process.in_edges.append(self)
            holder.out_edges.append(self)
        self.name = self.domain.name + "-->" + self.range.name
        self.security_context = None
        trans[self.domain][self.range] = self

    def not_obj(self, obj):
        #print str(self) + " for " + obj.name
        if self.domain == obj:
            return self.range
        elif self.range == obj:
            return self.domain
    def __repr__(self):
        return "<" + self.domain.name + "-->" + self.range.name + ">"

class NormalizedTransition:
    def __init__(self, process_context, action, holder_context, holder_type):
        self.action = action #'read','write', etc
        actobj = actions[action]
        self.direction = actobj.direction
        self.holder_type = holder_type
        self.evidence_for = []
        if self.direction == 'out': # writing
            self.domain_context = process_context
            self.range_context = holder_context
        elif self.direction == 'in': # reading
            self.domain_context = holder_context
            self.range_context = process_context
        normalized_trans[process_context + "--" + holder_context + "--" + self.action] = self
    def __repr__(self):
        type = self.holder_type
        if self.direction == 'out':
            begin = self.domain_context
            end = self.range_context
        else:
            begin = self.range_context
            end = self.domain_context
        return write_allow_instance(begin, end, type, self.action)
    def is_covered (self):
        coveredp = 0
        if self.domain_context == 'tbd' or self.range_context == 'tbd':
            return 0
        evidence = self.evidence_for
        for e in evidence:
            if Exists([m for m in e.meanings if m.recog.detection != 'violator']):
                coveredp = 1
                break
        return coveredp

def write_allow_instance (begin, end, type, action):
    return "AllowR('" + begin + "','" + end + "','" + type + "','" + action + "')"
    
def resource_factory (id, type, context = "default_context_for_resource"):
    """Creating a resource"""
    #strip quotes if necessary
    if id[0] == '"':
        id = id[1:-1]
    if id not in resources_dict.keys():
        #create a new one
        new = eval(type + "(id, id)")
        resources_dict[id] = new 
        new.security_context = context
        new.original_context = context
        if type != "Process":
            if context == "???":
                new.context_assignment_status = 'default'
            elif context != None:
                new.context_assignment_status = 'external'
            else: #None
                new.context_assignment_status = 'none'
    else: # have seen it
        new = resources_dict[id]
        if new.security_context == '???' and context != 'tbd':
            new.security_context = context
            new.original_context = context
    return resources_dict[id]
