""" /** Functions and Classes used in detecting instances of patterns

     *
     *  @author David Harris (DRH) - drh@mitre.org
     * 
     *

     *
     */"""


from util import *
from gen import Player

from specs import SPECIFIERS




################################# Dictionaries ########################################

classes = {}

def make_class_dictionary ():
    """ /** Take all the ClassObjects and place them in a dictionary accessible
         * by their name (represented as a string)
         *
         *   @author 
         *
         *   @return void 
         */
    """
    for item in get_class_objects():
        classes[item.name] = item
        
############################### Classes and Instances #####################################

#check definition of parameters
class Aggr:
    """ /** Aggr instances hold data on a set of classes that plays a role in a
         *  pattern's implementation         
         *  
         *   Parameters required for construction:
         *   @param label - role label used as index to aggregate of elements
         *   @param pattern - pattern recognizer
         *   @param elements - elements being gathered
         *   @author 
         */
    """
    def __init__(self, label, pattern, elements):
        self.label = label
        self.elements = [Player(s, label, -1, pattern, elements[s], {}) \
                         for s in elements.keys()]

def make_aggr(label, pattern, lst):
    """ /**    An Aggr 'factory' 
         *
         *   @author 
         *
         *   @return Aggr instance
         */
    """
    return Aggr(label, pattern, lst)
                         
class ClassObjectWrapper:
    """ /** A Dummy Class used as place holder for degenerate cases of 
         *  attributes holding a ClassObject         
         *  
         *   Parameters required for construction:
         *   @param name
         *   @param derived_from
         *   @param subclasses
         *   @param invoked_classes
         *   @author 
         */
    """
    def __init__(self, name = "None", derived_from = [],\
                 subclasses = [], invoked_classes = []):
        self.name = name
        self.file = 'unknown'
        self.line_start = "0"
        self._ID = "COW"
        self.derived_from = derived_from
        self.subclasses = subclasses
        self.class_of_methods_called = invoked_classes

#variables local to futil.py
""" cow is the singleton of ClassObjectWrapper - there will be one and only
one instance of ClassObjectWrapper"""
cow = ClassObjectWrapper()
cow.path = [""]
cow.has_patterns = []


############################ Accessors used in the specifications #########################

""" Next set of functions are used to create dictionaries whose keys
are classes and entries are sets of classes, methods, or locations"""

def collect_by_second (lst):
    """ /** a special utility that converts lists such as  [[a, x], [b, y], [c, y], [c, x]] to
                dictionary format with the second items as keys - {x : [a, c]],, y : [b,c]]]
         *   @author
         *
         *   @param lst - a list in the form shown above
         *
         *   @return dictionary format of list
         *
         */
    """
    ans = {}
    keys = seq2set([s[1] for s in lst])
    for s in lst:
        dict_put(ans, s[1], s[0])
    return ans


def return_types_of (lstMethods):
    """ /** dictionary where the keys are the return types of methods and the values are
            the methods 
         *   @author
         *
         *   @param lstMethods - a list  of MethodObjects
         *
         *   @return dictionary of types and methods 
         *
         */
    """
    return collect_by_second([[s, get_returned_type(s)] for s in lstMethods \
                                  if not get_returned_type(s) == None])
def nodes_in (lstClasses):
    """ /** dictionary where keys and values are the same class 
         *   @author
         *
         *   @param lstClasses - a list  of ClassObjects
         *
         *   @return dictionary of Classes (same class as key and value)
         *
         */
    """
    # degenerate structure created to match returns of other accessors
    #key is item itself
    dict = {}
    for s in lstClasses:
        dict[s] = s
    return dict    


def destinations_of (lstCalls):
    """ /** dictionary where keys are destination classes (A) and values are calls
            that target a method (m) of the destination class (e.g. A.m())
         *   @author
         *
         *   @param lstCalls - a list  of LocationObjects
         *
         *   @return dictionary of Class-method pairs
         *
         */
    """
    #should have location objects and key is destination class
    return collect_by_second([[s, get_range_of(s)] for s in lstCalls \
                              if not get_range_of(s) == cow])

def invocation_classes_of(lstCalls):
    """ dictionary where keys are classes of calling methods and values are calls within those methods"""
    #print lstCalls
    #should have location object and key is destination class
    return collect_by_second([[s, get_domain_of(s)] for s in lstCalls \
                              if not get_domain_of(s) == cow])      

def return_type_as_str(returntype):
    """ return the type associated with the object or string passed in"""
 #   x = None
    if is_str_stand_in(returntype):
        x= returntype
    else:
        if returntype != None:
            x = returntype.name
        else:
            x = None
    return x

def has_common_name(namelist,searchlist):
    """ return a boolean indicating that namelist and searchlist have a
    strings in common"""
    retvalue = 0
    for n in namelist:
        for s in searchlist:
            if s != None:
                if n.upper().find(s.upper()) >= 0:
                    retvalue = 1
                    return retvalue
    return retvalue

######################### Utilities used exclusively within find methods #######################



def next_classes_with_justifications (accessor_func, lstObj):
    """ Applies func to objs; returns a dictionary
    computed from a set of justifications (e.g., classes, methods, or locations) finds associated classes"""
    return apply(accessor_func,[lstObj])





