from util import *
from pres_util import *
from specs import SPECIFIERS
import config
import os
import role_fillers
import walk
import hreports
import pat2cls
import macro_defs
import rutil
from domain_model import NormalizedTransition, write_allow_instance
from pr import *
import sys
import home
import te_gen_util

class Meaning:
    def __init__(self, transobj, recogObj, domain_type, range_type):
        self.parent = transobj
        self.recog = recogObj
        self.pattern = recogObj.pattern.name
        self.detection = recogObj.detection
        self.used = 0
        self.domain_type = domain_type
        self.range_type = range_type
        self.domain = transobj.domain
        self.range = transobj.range
 
    def __repr__(self):
        return self.pattern + ": " + self.domain_type + "->" + self.range_type


def do_reports():
    populate_all_meanings()
    pat2cls.adjust_contexts()
    for k in trans.keys():
        for j in trans[k].keys():
            tranobj = trans[k][j]
##            print j
##            print k
            if tranobj.actobj.direction == 'out':
                p = tranobj.domain
                h = tranobj.range
            else:
                p = tranobj.range
                h = tranobj.domain
            if p.security_context != None and h.security_context != None:
                    key = p.security_context + "--" + h.security_context \
                          + "--" + tranobj.action
##                    print key
                    if  key not in normalized_trans.keys():
                        nt = NormalizedTransition(p.security_context, tranobj.action, h.security_context, h.__class__.__name__)
                    else:
                        nt = normalized_trans[key]
                    nt.evidence_for.append(tranobj)
    macro_defs.add_templates()
    gen_policy_spec()
##    gen_macros_report()
    gen_meanings_report()
##    gen_allows_report()
    gen_applications_report()
    gen_to_do()
    home.gen()
    rutil.init_reports()
    hreports.gen_recog_results_browser()
    hreports.gen_security_context_report()


   
def populate_meanings (node, recog):
    #Issue: may not know directionality of connection between node.focus and next
    role = walk.get_role(node)
    focus = node.focus
    if focus.__class__.__name__ == 'Transition':
        focus = focus.domain
    focus.roles.append([recog.pattern, role])
    if recog.pattern.distinguished_role == role:
        focus.is_distinguished_wrt.append(recog.pattern.name)
    
    for nr in node.with.keys():
        next_nodes = walk.next_nodes_of(node, nr)
        for nn in next_nodes:
            nfocus = nn.focus
            new_meaning = None
            if nfocus.__class__.__name__ == 'Transition':
                    nfocus = nfocus.domain
            if nfocus in trans[focus].keys():
                 transition  = trans[focus][nfocus]
                 new_meaning = Meaning(transition, recog, role, nr)
            elif node.focus in trans[nfocus].keys():
                transition = trans[nfocus][focus]
                new_meaning = Meaning(transition, recog, nr, role)
            if new_meaning != None:
                transition.meanings.append(new_meaning)
            populate_meanings(nn, recog)

def gen_to_do():
    prgm = get_current_program()
    pname = prgm.name
    fileid  = 'ToDo.html'
    fileStr = config.result_file(fileid)
    out = open(fileStr, "w")
    print "Writing to " + fileStr
    out.write('<hr>')
    no_context_resources = [x for x in resources_dict.values() if x.security_context in ['???',  'tbd']]
    if len(no_context_resources) > 0:
        out.write('<h3>Possible occurrance of failed attempt to open a file.</h3>\n')
        for r in no_context_resources:
            out.write('<br>May want to assign security context to ' + obj2href(r) + '. \n<hr>')
    multi_assignment = [x for x in resources_dict.values() if x.context_assignment_status == 'problematic']
    if len(multi_assignment) > 0:
        out.write('<h3>More than one constraint on type generation for resources</h3> \n')
        for r in multi_assignment:
            out.write('<br> More than one constraint on ' + resource2href(r, 'PolicyHelp.html') + '.\n')
            out.write('Verify or modify security context. \n')
        out.write('<hr>')
    new = [x for x in resources_dict.values() if x.security_context != x.original_context]
    if len(new) > 0:
        out.write('<h3> Other Polgen type assignments </h3> \n')
        for r in new:
            out.write('<br> spar assigned security_context to ' \
                      + resource2href(r, 'PolicyHelp.html') + '.  Verify or modify. \n')                  
    out.close()

def gen_policy_spec():
    te_gen_util.pat_instances = []
    prgm = get_current_program()
    pname = prgm.name
    fileid  = pname + "_policy_spec.py"
    fileStr = config.result_file(fileid)
    out = open(fileStr, "w")
    print "Writing to " + fileStr
    out.write('from util import * \n')
    out.write('from pr import * \n')
    out.write('"""This file contains policy statement assertions along with comments that document underlying track \
evidence.  For some pattern instances, resources participating in the pattern have been assigned a new \
type so that pattern constraints can be maintained. For each pattern instance the policy \
author may take one of three actions - accept the assertion as is, select individual "allows" using the \
automatically gnenerated types , or select individual "allows" using the original types.  The author should \
make sure that the desired assertions are uncommented. """\n')
    out.write('############ New Types #################### \n\n')
    out.write('# If any of these are removed, also replace deleted type in any of the assertions below \n')
    for r in resources_dict.values():
        if r.original_context != r.security_context:
            out.write('Declare("' + r.security_context + '","' \
                      + r.__class__.__name__.lower()  \
                      + '_type") # original was ' + r.original_context + ' \n') 
    out.write(" \n################### Individual Allows ################# \n\n")
    for nt in normalized_trans.values():
        #note spec might not contain all instances of evidence
        if not nt.is_covered():
            for e in nt.evidence_for:
                out.write('# ' + str(e) + '\n')
            if nt.action != 'spawn':
                if nt.action in ['activate', 'exec']:
                    out.write("CanExecR('" + nt.domain_context + "','" \
                              + nt.range_context + "') \n")
                else:
                    out.write(str(nt) + '\n\n')
    for p in [p for p in resources_dict.values() if p.__class__.__name__ == 'Process']:
        execs = [x.range for x in p.out_edges if x.action == 'exec']
        targets = [p.security_context]
        target_processes = []
        for t in execs:
            targets.append(t.security_context)
            target_processes.append(t)
        targets = seq2set(targets)
        if len(target_processes) > 0:
            out.write('# ' + p.name + ' execs ' + str([tp.name for tp in target_processes]) + '\n')
            outstr = 'CanExecR('
            for t in targets:
                outstr = outstr + "'" + t + "'" + ','
            out.write(outstr[:-1] + ') \n')
        
    out.write("##################### Pattern Instances #################### \n\n")
    out.write("# Uncomment individual assertions or leave as is for entire pattern \n")
    for recog in SPECIFIERS:
        if len(PRGM.pattern_instances[recog.__name__]) != 0:
            out.write('# ' + recog.pattern.name + ' instances observed by ' + recog.__name__ + '\n')
            count = 1
            for instance in PRGM.pattern_instances[recog.__name__]:
                out.write('# count is ' + str(count) + '\n')
                count = count + 1
                gen_pattern_instances(out, recog, instance)
                out.write(' \n')
    out.write('gen_te()')
    out.close()



def create_patternR(recog, individual):
    individual.append(0)
    pattern = recog.pattern
    genstr = eval(pattern.name + 'R')
    return apply(genstr, individual)

def gen_pattern_instances(out, recog, instance):
    """ Generate a set of macro statements - one for each path thru the instance tree"""
    for individual in walk.instance2individuals(instance):
        pr = create_patternR(recog, individual)
        if len(pr.elements) > 0:
            out.write('#Alternatives - \n')
            for e in pr.elements:
                out.write("#" + e.spec() + "\n")
            out.write('# or \n')
        #print pr
        out.write(pr.spec())
        


def populate_all_meanings ():
    for recog in SPECIFIERS:
        sname = recog.__name__
        instances = PRGM.pattern_instances[sname]
        for node in instances:
            populate_meanings(node, recog)
    

def display_meanings ():
    for r in [r for r in resources_dict.values() if r.__class__.__name__ != 'Process']:
        print "--"
        print r.name
        for t in r.out_edges:
            print "allow " + t.range.name + " " + r.name \
                      + ":" \
                      + r.__class__.__name__ + " " + t.action
            print "   because " + str(t.meanings)
        for t in r.in_edges:
            print "allow " + t.domain.name + " " + r.name + ":" \
                      + r.__class__.__name__ + " " + t.action
            print "   because " + str(t.meanings)



def meanings2html (strm):
    strm.write('<table border= 3><tr><td>')
    for r in [x for x in resources_dict.values()]:
        strm.write('<a href = #' + r.name + '>' + r.name + \
                   '</a>---')
    strm.write('</td></tr></table>')
    for r in [x for x in resources_dict.values()]:
        strm.write('<hr><hr>')
        cname = r.__class__.__name__
        context = r.security_context
        strm.write('<a name = ' + str(r.id) + '><h3>' \
                   + r.name + " a " + cname + '</h3> \n')
        strm.write('with security context =  <font color = ff00ff><a href=Security-Context.html#' \
                   + str(context) + ">" + str(context) + "</a></font><br>\n")
        for its_role in r.roles:
            role = its_role[1]
            pattern = its_role[0]
            #print pattern.name
            strm.write("<br>Playing " + role + " role for a " \
                       + "<a href=Pattern-Instances.html#" + pattern.name + \
                       ">" + pattern.name + "</a> \n")
            if role in pattern.to_avoid.keys():
                strm.write(pattern.to_avoid[role].display(r))
        if r.__class__.__name__ != 'Process':
            for t in r.out_edges:
                m = "<br>--- no obvious pattern"
                if len(t.meanings) > 0:
                    m = "<br>----   because "
                    for meaning in t.meanings:
                        m = m + handle_meaning(meaning, 'out')
                strm.write("<br><table border = 2><tr><td bgcolor = ffff44>allow  " + resource2href(t.range) + "  " + str(context) \
                          + ": " \
                          + r.__class__.__name__ + "  " + t.action + '</td></tr></table> \n')
                strm.write(m)
            for t in r.in_edges:
                m = "<br>--- no obvious pattern"
                if len(t.meanings) > 0:
                    m = "<br>----   because "
                    for meaning in t.meanings:
                        m = m + handle_meaning(meaning, 'in')
                strm.write("<br><table border = 2><tr><td bgcolor = ffff44>allow  " + resource2href(t.domain) + "  " + \
                               str(context) + ": " \
                          + r.__class__.__name__ + "  " + t.action + '</td></tr></table> \n')
                strm.write(m)
                
def handle_meaning(meaningObj, direction = 'in'):
    "Given a meaning returns a string"
    if direction == 'out':
        whatami = meaningObj.domain_type
        partner_role = meaningObj.range_type
        partner = meaningObj.range.name
    else:
        whatami = meaningObj.range_type
        partner_role = meaningObj.domain_type
        partner = meaningObj.domain.name
    return "<br>---- playing " + whatami + " role in an instance of " + meaningObj.pattern + \
           " and connecting to " + partner + ":" + partner_role 
    
    
def gen_meanings_report ():
    """ creates web-page showing all creation events"""
    prgm = get_current_program()
    pname = prgm.name
    if pname == "":
        pname = "e-commerce" # temp hack
    fileid  = "PolicyHelp.html"
    fileStr = os.path.join(config.results_directory, fileid)
    out = open(fileStr, "w")
    print "Writing to " + fileStr
    top_of_file(out, "Policy Authoring for " + capitalize_it(pname))
    out.write("<br> Go to <a href=Pattern-Instances.html> Pattern Instance Report </a> \n")
    meanings2html(out)
    out.close()

def gen_applications_report ():
    prgm = get_current_program()
    pname = prgm.name
    fileid  = "Applications.html"
    fileStr = config.result_file(fileid)
    out = open(fileStr, "w")
    print "Writing to " + fileStr
    top_of_file(out, 'Applications and Processes for ' + capitalize_it(pname))
    for app in applications:
        out.write('<h3>' + app.name + '</h3><br> \n')
        out.write('<br> From file ' + app.name + '.tracked \n')
        out.write('<br> Processes \n')
        for p in app.processes.keys():
            out.write('<br>---- ' + resource2href(app.processes[p], 'PolicyHelp.html') + '\n') 
    out.close()
                     
def gen_macros_report ():
    prgm = get_current_program()
    pname = prgm.name
    fileid  = "Macros.txt"
    fileStr = config.result_file(fileid)
    out = open(fileStr, "w")
    print "Writing to " + fileStr
    for recog in SPECIFIERS:
        if recog.__name__ in  PRGM.pattern_instances.keys():
            if recog.detection != 'violator':
                for instance in PRGM.pattern_instances[recog.__name__]:
                     gen_instance_macro(out, recog, instance)
    out.close()

def gen_instance_macro(strm, recog, instance):
    """ Generate a macro statement """
    pat = recog.pattern
    print pat
    template = pat.template
    if template != None:
        #pattern has been defined
        tname = template.name
        troles = template.roles
        nbr_roles = len(troles)
        for individual in walk.instance2individuals(instance):
            #generate a line
            strm.write(tname + '(')
            count = 1
            for p in individual:
                if p.security_context != None:
                    strm.write(p.security_context)
                else:
                    strm.write("unknown")
                count = count + 1
                if count != nbr_roles:
                    strm.write(', ')
            strm.write(') \n\n')

            
    
def gen_allows_report ():
    """ creates ascii text showing all allowed events"""
    prgm = get_current_program()
    pname = prgm.name
    if pname == "":
        pname = "unknown" 
    fileid  = pname + ".te"
    fileStr = os.path.join(config.results_directory , fileid)
    out = open(fileStr, "w")
    print "Writing to " + fileStr
    allows_to_txt(out)
    out.close()

def allows_to_txt (strm):
    for r in [x for x in resources_dict.values()]:
##        for its_role in r.roles:
##            role = its_role[1]
##            pattern = its_role[0]
##            #print pattern.name
##            strm.write("Playing " + role + " role for a " + pattern.name + " \n")
##            if role in pattern.to_avoid.keys():
##                strm.write(pattern.to_avoid[role].display(r))
        if r.__class__.__name__ == 'Process':
            for t in r.out_edges:
                if t.range.security_context != None:
                    m = "#---- no obvious pattern \n"
                    if len(t.meanings) > 0:
                        m = "#----   " + str(t.meanings) + " \n"
                    strm.write(m)
##                    print "  -- "
##                    print r.name
##                    print r.security_context
##                    print t.range.name
##                    print t.range.security_context
                    strm.write('#---- evidence = ' + r.name + ' to ' + t.range.name + \
                               "\n")
                    strm.write("allow  " + r.security_context + "  " + t.range.security_context \
                              + ": " \
                              + t.range.__class__.__name__ + "  " \
                               + t.action + "\n\n")
                else:
                    connects_to = [x.range for x in t.range.out_edges][0]
                    strm.write("allow " + r.security_context + "  " + \
                               connects_to.security_context +"  " + \
                               t.action + "\n\n")
                
            for t in r.in_edges:
                if t.domain.security_context != None:
                    m = "#---- no obvious pattern \n"
                    if len(t.meanings) > 0:
                        m = "#----   " + str(t.meanings) + " \n"
                    strm.write(m)
                    strm.write('#---- evidence = ' + r.name + ' to ' + t.domain.name + \
                               '\n')
                    strm.write("allow  " + r.security_context + "  " + \
                                   t.domain.security_context + ": " \
                              + t.domain.__class__.__name__ + "  " \
                               + t.action + "\n\n")
        
